/* PDF dumper and other examples based on PDFlib PLOP
 *
 * Version for Android, derived from standard dumper.java
 */

package com.example.plop_demo;

import android.content.Context;
import android.support.v7.app.AppCompatActivity;
import com.pdflib.PLOPException;
import com.pdflib.plop;

import java.io.*;
import android.app.Activity;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

public class dumper extends AppCompatActivity {

    private static void add_asset_to_pvf(Context context, plop plop, String filename)
            throws IOException, PLOPException {
        InputStream stream = context.getAssets().open(filename);
        byte[] imageData = new byte[stream.available()];
        stream.read(imageData);
        stream.close();
        plop.create_pvf(filename, imageData, "");
    }

    static final String infilename = "PLOP-datasheet.pdf";
    static final String docoptlist = "requiredmode=minimum";
    static final String globaloptlist = "";

    static plop plop = null;

    // static final String searchpath = "/data/data/com.pdflib";

    public static String dumper(Context context) {
        String result = "";
        String out = "";
        int count, pcosmode;
        int i, doc;
        boolean plainmetadata;
        String objtype;
        String optlist;

        try
        {
            plop = new plop();
            //plop.set_option("logging={filename=/data/data/com.pdflib/trace.txt remove}");

	    /*
             * Inputfiles etc are stored as assets
             * to use these resources we load them to
             * the PDFlib Virtual File System (PVF).
             */
            add_asset_to_pvf(context, plop, infilename);

            // optlist = "searchpath={" + searchpath + "}";
            // plop.set_option(optlist);

            plop.set_option(globaloptlist);

            if ((doc = plop.open_document(infilename, docoptlist)) == -1) {
                throw new Exception("ERROR " + plop.get_errmsg());
            }

            /* --------- general information (always available) */

            pcosmode = (int) plop.pcos_get_number(doc, "pcosmode");

            result += "       File name: " + plop.pcos_get_string(doc, "filename") + "\n";

            result += "     PDF version: " + plop.pcos_get_string(doc, "pdfversionstring") + "\n";

            result += "      Encryption: " + plop.pcos_get_string(doc, "encrypt/description") + "\n";

            result += "       Master pw: " + ((plop.pcos_get_number(doc, "encrypt/master") != 0) ? "yes" : "no") + "\n";

            result += "         User pw: " +
                    ((plop.pcos_get_number(doc, "encrypt/user") != 0) ? "yes" : "no") + "\n";

            result += "    Text copying: " +
                    ((plop.pcos_get_number(doc, "encrypt/nocopy") != 0) ? "no" : "yes") + "\n";

            result += "      Linearized: " +
                    ((plop.pcos_get_number(doc, "linearized") != 0) ? "yes" : "no") + "\n";

            if (pcosmode == 0) {
		throw new Exception(
                        "Minimum mode: no more information available\n" + "\n");
            }

            /* --------- more details (requires at least user password) */

            result += "    PDF/X status: " + plop.pcos_get_string(doc, "pdfx") + "\n";

            result += "    PDF/A status: " + plop.pcos_get_string(doc, "pdfa") + "\n";
            result += "   PDF/UA status: " + plop.pcos_get_string(doc, "pdfua") + "\n";
            result += "   PDF/VT status: " + plop.pcos_get_string(doc, "pdfvt") + "\n";

            boolean xfa_present =
                    !plop.pcos_get_string(doc, "type:/Root/AcroForm/XFA").equals("null");
            result += "        XFA data: " + (xfa_present ? "yes" : "no") + "\n";

            result += "      Tagged PDF: " +
                    ((plop.pcos_get_number(doc, "tagged") != 0) ? "yes" : "no") + "\n" + "\n";

                /* --------- analyze digital signatures */

            count = (int) plop.pcos_get_number(doc, "length:signaturefields");

            for (i = 0; i < count; i++) {
                String sigtype;
                String sigfields;

                sigfields = "signaturefields[" + i + "]/fullname";
                result += "  Sig. field #" + i + 1 + ": '"
                        + plop.pcos_get_string(doc, sigfields) + "'," + "\n";

                sigfields = "signaturefields[" + i + "]/visible";
                result += " "
                        + ((int) plop.pcos_get_number(doc, sigfields) != 0 ? "visible" : "invisible") + "\n";

                sigfields = "signaturefields[" + i + "]/sigtype";
                sigtype = plop.pcos_get_string(doc, sigfields);

                if (sigtype.equals("none")) {
                    result += ", unsigned\n" + "\n";
                    continue;
                }

                result += " " + sigtype + " signature" + "\n";

                if (sigtype.equals("certification")) {
                    sigfields = "signaturefields[" + i + "]/permissions";
                    result += " (" + plop.pcos_get_string(doc, sigfields) + " allowed)" + "\n";
                }

                sigfields = "signaturefields[" + i + "]/cades";
                if ((int) plop.pcos_get_number(doc, sigfields) != 0)
                    result += ", CAdES";

                result += "\n";
            }
            if (count > 0) {
                result += "No. of revisions: "
                        + (int) plop.pcos_get_number(doc, "revisions") + "\n\n";
            }

            result += "  Reader-enabled: "
                    + plop.pcos_get_string(doc, "usagerights") + "\n";

            result += "    No. of pages: " +
                    (int) plop.pcos_get_number(doc, "length:pages") + "\n";

            result += "     Page 1 size: width=" +
                    (int) plop.pcos_get_number(doc, "pages[0]/width") + ", height=" +
                    (int) plop.pcos_get_number(doc, "pages[0]/height") + "\n";

            count = (int) plop.pcos_get_number(doc, "length:fonts");
            result += "    No. of fonts: " + count + "\n";

            for (i = 0; i < count; i++) {
                String fonts;

                fonts = "fonts[" + i + "]/embedded";
                if (plop.pcos_get_number(doc, fonts) != 0)
                    result += "embedded ";
                else
                    result += "unembedded ";

                fonts = "fonts[" + i + "]/type";
                result += plop.pcos_get_string(doc, fonts) + " font ";
                fonts = "fonts[" + i + "]/name";
                result += plop.pcos_get_string(doc, fonts);
            }

            result += "\n";

            plainmetadata = plop.pcos_get_number(doc, "encrypt/plainmetadata") != 0;

            if (pcosmode == 1 && !plainmetadata &&
                    plop.pcos_get_number(doc, "encrypt/nocopy") != 0) {
                result += "Restricted mode: no more information available" + "\n";
                plop.delete();
                return result;
            }

            /* ----- document info keys and XMP metadata (requires master pw) */

            count = (int) plop.pcos_get_number(doc, "length:/Info");

            for (i = 0; i < count; i++) {
                String info;
                String key;
                int len;

                info = "type:/Info[" + i + "]";
                objtype = plop.pcos_get_string(doc, info);

                info = "/Info[" + i + "].key";
                key = plop.pcos_get_string(doc, info);
                len = 16 - key.length();
                while (len-- > 0) result += " ";

                result += key + ": ";

                /* Info entries can be stored as string or name objects */
                if (objtype.equals("name") || objtype.equals("string")) {
                    info = "/Info[" + i + "]";
                    result += "'" + plop.pcos_get_string(doc, info) + "'";
                } else {
                    info = "type:/Info[" + i + "]";
                    result += "(" + plop.pcos_get_string(doc, info) +
                            " object)";
                }
            }

            result += "\n" + "   XMP meta data: ";

            objtype = plop.pcos_get_string(doc, "type:/Root/Metadata");
            if (objtype.equals("stream")) {
                byte[] contents;
                int len;

                contents = plop.pcos_get_stream(doc, "", "/Root/Metadata");
                result += contents.length + " bytes ";
                result += "";
            } else {
                result += "not present";
            }

            plop.close_document(doc, "");
        } catch (PLOPException e) {
            result = "PLOP exception occurred in dumper sample:" +
                    "[" + e.get_errnum() + "] " + e.get_apiname() +
                    ": " + e.get_errmsg();
        } catch (Exception e) {
            result = "Exception: " + e.toString();
        } finally {
            if (plop != null) {
                plop.delete();
            }
        }
        return result;
    }
}
